require 'spec_helper'

describe "packagecloud debtools parsers" do
  describe "DEB files" do
    before do
      @deb = PackagecloudDebtools::Parser.parse_from_file("spec/fixtures/jake_1.0-7_amd64.deb")
    end

    it "has data" do
      expect(@deb).not_to be_nil
      expect(@deb.fields).not_to be_nil
      expect(@deb.fields.length).to be > 0
      expect(@deb.fields['files'].length).to eq(9)
      expect(@deb.fields['size']).to eq(3290)
    end

    it "has basic deb attributes" do
      expect(@deb.fields['package']).to eq("jake")
      expect(@deb.fields['version']).to eq("1.0-7")
      expect(@deb.fields['architecture']).to eq("amd64")
      expect(@deb.fields['maintainer']).to eq("jake <joe@packagecloud.io>")
      expect(@deb.fields['installed-size']).to eq("39")
      expect(@deb.fields['section']).to eq("misc")
      expect(@deb.fields['priority']).to eq("optional")
      expect(@deb.fields['description']).to eq("На берегу пустынных волн\n")

      expect(@deb.fields['relationships']['depends']).to eq("dpkg (>= 1.15.4) | install-info, libc6 (>= 2.4)")
      expect(@deb.fields['relationships']['provides']).to eq("jake")
      expect(@deb.fields['relationships']['replaces']).to eq("gnome-games (<< 1:2.27.5-0ubuntu2), gnome-games-data (<< 1:2.27.5-0ubuntu2)")
      expect(@deb.fields['relationships']['breaks']).to eq("gnome-control-center (<< 1:2.30.0-0ubuntu4)")
      expect(@deb.fields['relationships']['suggests']).to eq("indent")
      expect(@deb.fields['relationships']['conflicts']).to eq("cervisia-kde4, cheese (<< 2.29.90-0ubuntu1)")
      expect(@deb.fields['homepage']).to eq("http://www.chkrootkit.org/")
      expect(@deb.fields['origin']).to eq("Ubuntu")
      expect(@deb.fields['bugs']).to eq("https://bugs.launchpad.net/ubuntu/+filebug")
      expect(@deb.fields['description_md5']).to eq("659efd19f04d6465b7f5821948bb0952")
      expect(@deb.fields['md5sum']).to eq("8d47647f97d78e6bd4428afa93e67d76")
      expect(@deb.fields['sha1sum']).to eq("ce1c52b7d1a4abc44e54a3f4bab28b9f92b5bc06")
      expect(@deb.fields['sha256sum']).to eq("84ce72ad3627539f15176f16004f6ed95bed4efcbe5f591bf3e0c50031826892")
      expect(@deb.fields['sha512sum']).to eq("f4c99e53490bd98ee3cd57224a5d0a3bf351d1ddc160cff56d1736a06db80c40d28ac104009cc76dbdaab7beb65a07cb6b712ad3d8bb3471d8bd289fd3d5570e")
      filenames = @deb.fields['files'].inject([]) do |memo, v|
        memo << v[:path]
      end

      expect(filenames.length).to eq(9)
      ["/usr/", "/usr/share/", "/usr/share/doc/", "/usr/share/doc/jake/", "/usr/share/doc/jake/copyright", "/usr/share/doc/jake/changelog.Debian.gz", "/usr/local/", "/usr/local/bin/", "/usr/local/bin/jake"].each do |entry|
        expect(filenames.include?(entry)).to eq(true)
      end
    end

    it "generates the proper md5 sum for long descriptions" do
      actual_desc = "GNU C++ compiler (multilib files)\n This is the GNU C++ compiler, a fairly portable optimizing compiler for C++.\n .\n On architectures with multilib support, the package contains dependencies\n for the non-default multilib architecture(s).\n"
      @long_desc_deb = PackagecloudDebtools::Parser.parse_from_file("spec/fixtures/g++-multilib_4.6.3-1ubuntu5_amd64.deb")
      expect(@long_desc_deb.fields['description_md5']).to eq('9df28aeef12fea2f44664b1d4c3bac30')
      expect(@long_desc_deb.fields['description']).to eq(actual_desc)
    end
  end

  describe "changes files" do
    before do
      @changes = PackagecloudDebtools::Parser.parse_from_file("spec/fixtures/jake_1.0-7_amd64.changes")
    end

    it "has data" do
      expect(@changes).not_to be_nil
      expect(@changes.fields).not_to be_nil
      expect(@changes.fields.length).to be > 0
      expect(@changes.fields['size']).to eq(1465)
    end

    it "has basic changes attributes" do
      expect(@changes.fields['format']).to eq("1.8")
      expect(@changes.fields['date']).to eq("Wed, 11 Jun 2014 12:29:43 -0800")
      expect(@changes.fields['source']).to eq("jake")
      expect(@changes.fields['binary']).to eq("jake jake-doc")
      expect(@changes.fields['architecture']).to eq("source amd64 all")
      expect(@changes.fields['version']).to eq("1.0-7")
      expect(@changes.fields['distribution']).to eq("precise")
      expect(@changes.fields['urgency']).to eq("low")
      expect(@changes.fields['maintainer']).to eq("jake <joe@packagecloud.io>")
      expect(@changes.fields['changed-by']).to eq("Joe Damato <joe@packagecloud.io>")
      expect(@changes.fields['description']).to eq(["jake       - На берегу пустынных волн", "jake-doc   - Documentation for jake"])
      expect(@changes.fields['changes']).to eq(["jake (1.0-7) precise; urgency=low", ".", "* Add build and install deps, conflicts, breaks for more thorough testing"])
    end

    it "has the correct checksum and file objects" do
      [ 'checksums-sha1', 'checksums-sha256' ].each do |type|
        fileinfo = @changes.fields[type].inject([]) do |fileinfo, entry|
          fileinfo << [entry.filename, entry.size]
        end
        expect(fileinfo).to eq([["jake_1.0-7.dsc", 1057], ["jake_1.0-7.debian.tar.gz", 1552], ["jake_1.0-7_amd64.deb", 3290], ["jake-doc_1.0-7_all.deb", 1842]])
      end

      fileinfo = @changes.fields['files'].inject([]) do |fileinfo, entry|
        fileinfo << [entry.filename, entry.size, entry.section, entry.priority]
      end
      expect(fileinfo).to eq([["jake_1.0-7.dsc", 1057, "misc", "optional"], ["jake_1.0-7.debian.tar.gz", 1552, "misc", "optional"], ["jake_1.0-7_amd64.deb", 3290, "misc", "optional"], ["jake-doc_1.0-7_all.deb", 1842, "doc", "optional"]])
    end

    it "can verify checksums of all mentioned files" do
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        @changes.fields[type].each do |csum|
          expect(csum.verify).to eq(true)
        end
      end
    end

    it "can return an array of files" do
      expect(@changes.files).not_to be_nil
      expect(@changes.files.count).to eq(4)
      expect(@changes.files).to eq([{:filename=>"jake_1.0-7.dsc",
                                     :md5sum=>"b16ec286554b02656e6993b5545b7867",
                                     :section=>"misc",
                                     :priority=>"optional",
                                     :size=>1057,
                                     :sha1sum=>"8c9972785ef77ced4586be14c0f8a80fbabcef19",
                                     :sha256sum=>
                                      "7a8bbe60c5bd3522181ed82f730c875a8d3e76fa664bde7fe809a0b90939fcba"},
                                    {:filename=>"jake_1.0-7.debian.tar.gz",
                                     :md5sum=>"c953eb671e91c965768a9b436f9375b7",
                                     :section=>"misc",
                                     :priority=>"optional",
                                     :size=>1552,
                                     :sha1sum=>"c6600e1d0a8e5aee5de18e596b52fb8a3340ba83",
                                     :sha256sum=>
                                      "c7455f1069cec79574999d8c7309c395bec7866d3a0a2daad42c0ce3924f1794"},
                                    {:filename=>"jake_1.0-7_amd64.deb",
                                     :md5sum=>"8d47647f97d78e6bd4428afa93e67d76",
                                     :section=>"misc",
                                     :priority=>"optional",
                                     :size=>3290,
                                     :sha1sum=>"ce1c52b7d1a4abc44e54a3f4bab28b9f92b5bc06",
                                     :sha256sum=>
                                      "84ce72ad3627539f15176f16004f6ed95bed4efcbe5f591bf3e0c50031826892"},
                                    {:filename=>"jake-doc_1.0-7_all.deb",
                                     :md5sum=>"23e7c23ed08ba0763adfa2a6377869cc",
                                     :section=>"doc",
                                     :priority=>"optional",
                                     :size=>1842,
                                     :sha1sum=>"95531529a37381acf594598e4a5ce0f81a28a7e6",
                                     :sha256sum=>
                                      "2156a53013483a99b690d2860cb4a9144a4c9406b4680b835b36172a6e1dd841"}])
    end
  end

  describe "gpg signed dsc file" do
    before do
      @dsc = PackagecloudDebtools::Parser.parse_from_file("spec/fixtures/redis_2.2.12-1build1.dsc")
    end

    it "has data" do
      expect(@dsc).not_to be_nil
      expect(@dsc.fields).not_to be_nil
      expect(@dsc.fields.length).to be > 0
      expect(@dsc.fields['size']).to eq(1211)
    end

    it "has dsc attributes" do
      expect(@dsc.fields['format']).to eq("3.0 (quilt)")
      expect(@dsc.fields['source']).to eq("redis")
      expect(@dsc.fields['binary']).to eq("redis-server, redis-doc")
      expect(@dsc.fields['architecture']).to eq("any all")
      expect(@dsc.fields['version']).to eq("2:2.2.12-1build1")
      expect(@dsc.fields['origin']).to eq(nil)
      expect(@dsc.fields['maintainer']).to eq("Chris Lamb <lamby@debian.org>")
      expect(@dsc.fields['dm-upload-allowed']).to eq(nil)
      expect(@dsc.fields['homepage']).to eq("http://redis.io/")
      expect(@dsc.fields['standards-version']).to eq("3.9.2")
      expect(@dsc.fields['vcs']).to eq("git")
      expect(@dsc.fields['vcs-url']).to eq("git://github.com/lamby/pkg-redis.git")
      expect(@dsc.fields['vcs-browser']).to eq("https://github.com/lamby/pkg-redis")
      expect(@dsc.fields['relationships']['build-depends']).to eq("debhelper (>= 7.0.50)")
      expect(@dsc.fields['relationships']['build-depends-indep']).to eq(nil)
      expect(@dsc.fields['relationships']['build-conflicts']).to eq(nil)
      expect(@dsc.fields['package-list'].length).to eq(2)
    end

    it "has a correct package-list" do
      packagelist = @dsc.fields['package-list'].inject([]) do |memo, item|
        memo << [ item.name, item.type, item.section, item.priority ]
      end
      expect(packagelist).to eq([["redis-doc", "deb", "doc", "optional"], ["redis-server", "deb", "database", "optional"]])
    end

    it "has the correct files and checksum objects" do
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        fileinfo = @dsc.fields[type].inject([]) do |fileinfo, entry|
          fileinfo << [entry.filename, entry.size]
        end
        expect(fileinfo).to eq([["redis_2.2.12.orig.tar.gz", 455240], ["redis_2.2.12-1build1.debian.tar.gz", 16930]])
      end
    end

    it "can verify checksums and files" do
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        @dsc.fields[type].each do |csum|
          expect(csum.verify).to eq(true)
        end
      end
      expect(@dsc.verify).to eq(true)
    end

    it "condenses files and sums" do
      expect(@dsc.files).to eq([
                               {:filename=>"redis_2.2.12.orig.tar.gz",
                                :md5sum=>"61b287860cab3e89c49390b8f5ffb697",
                                :size=>455240,
                                :sha1sum=>"66d44756e0f3371d9e37b521f5cc17e12e1efba3",
                                :sha256sum=>"9e2a4133b920fc4eed731c9cd3a9d0b60e701179ea98b2944ab434e7b1d0db03"},
                               {:filename=>"redis_2.2.12-1build1.debian.tar.gz",
                                :md5sum=>"0868468690f99f539ffad87bea4e3210",
                                :size=>16930,
                                :sha1sum=>"85ba0ffa2868d6fd905033d1aa664791285bc490",
                                :sha256sum=>"6a92439f55da11163b88e8c7617e0c3a09b2c73e3ac5ae1deed6ebef68fc7b6a"},
                               {:filename=>"redis_2.2.12-1build1.dsc",
                                :size=>1211,
                                :md5sum=>"d47f7e61c63b8416dcbdf8b5711dd717",
                                :sha1sum=>"097ef316ce50fccfded714a88d07dcd04fbaee39",
                                :sha256sum=>"c4dc5a350bd094377f88d9727a45302dd98f958fd30f53bebac40698cf4aa071"}])
    end
  end

  describe "dsc files" do
    before do
      @dsc = PackagecloudDebtools::Parser.parse_from_file("spec/fixtures/jake_1.0-7.dsc")
    end

    it "has data" do
      expect(@dsc).not_to be_nil
      expect(@dsc.fields).not_to be_nil
      expect(@dsc.fields.length).to be > 0
      expect(@dsc.fields['size']).to eq(1057)
    end

    it "has dsc attributes" do
      expect(@dsc.fields['format']).to eq("3.0 (quilt)")
      expect(@dsc.fields['source']).to eq("jake")
      expect(@dsc.fields['binary']).to eq("jake, jake-doc")
      expect(@dsc.fields['architecture']).to eq("i386 amd64 all")
      expect(@dsc.fields['version']).to eq("1.0-7")
      expect(@dsc.fields['origin']).to eq("Ubuntu")
      expect(@dsc.fields['maintainer']).to eq("jake <joe@packagecloud.io>")
      expect(@dsc.fields['dm-upload-allowed']).to eq("yes")
      expect(@dsc.fields['homepage']).to eq("http://www.chkrootkit.org/")
      expect(@dsc.fields['standards-version']).to eq("3.9.3")
      expect(@dsc.fields['vcs']).to eq("git")
      expect(@dsc.fields['vcs-url']).to eq("git://git.debian.org/git/collab-maint/chkrootkit.git")
      expect(@dsc.fields['vcs-browser']).to eq("http://git.debian.org/?p=collab-maint/chkrootkit.git")
      expect(@dsc.fields['relationships']['build-depends']).to eq("pkg-config, quilt")
      expect(@dsc.fields['relationships']['build-depends-indep']).to eq("debhelper (>= 2.0.40), gettext, bzip2, cpio")
      expect(@dsc.fields['relationships']['build-conflicts']).to eq("tk8.0-dev")
      expect(@dsc.fields['package-list'].length).to eq(2)
    end

    it "has a correct package-list" do
      packagelist = @dsc.fields['package-list'].inject([]) do |memo, item|
        memo << [ item.name, item.type, item.section, item.priority ]
      end
      expect(packagelist).to eq([["jake", "deb", "misc", "optional"], ["jake-doc", "deb", "doc", "optional"]])
    end

    it "has the correct files and checksum objects" do
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        fileinfo = @dsc.fields[type].inject([]) do |fileinfo, entry|
          fileinfo << [entry.filename, entry.size]
        end
        expect(fileinfo).to eq([["jake_1.0.orig.tar.bz2", 2438], ["jake_1.0-7.debian.tar.gz", 1552]])
      end
    end

    it "can verify checksums and files" do
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        @dsc.fields[type].each do |csum|
          expect(csum.verify).to eq(true)
        end
      end
      expect(@dsc.verify).to eq(true)
    end

    it "condenses files and sums" do
      expect(@dsc.files).to eq([{filename: "jake_1.0.orig.tar.bz2",
                                 md5sum: "82f012c9100501cd839b970e1d7da3c8",
                                 size: 2438,
                                 sha1sum: "604091fa5d4dc059c46255a133cc4a5b06a04d01",
                                 sha256sum: "421dfae1142ece6e56b63398198c6a610d3fe3ae21b07543fa15638eee5d7ca3"},
                                {filename: "jake_1.0-7.debian.tar.gz",
                                 md5sum: "c953eb671e91c965768a9b436f9375b7",
                                 size: 1552,
                                 sha1sum: "c6600e1d0a8e5aee5de18e596b52fb8a3340ba83",
                                 sha256sum: "c7455f1069cec79574999d8c7309c395bec7866d3a0a2daad42c0ce3924f1794"},
                                {filename: "jake_1.0-7.dsc",
                                 size: 1057,
                                 md5sum: "b16ec286554b02656e6993b5545b7867",
                                 sha1sum: "8c9972785ef77ced4586be14c0f8a80fbabcef19",
                                 sha256sum: "7a8bbe60c5bd3522181ed82f730c875a8d3e76fa664bde7fe809a0b90939fcba"])
    end
  end
end

